@@ -3,17 +3,18 @@
 !
 !  Supplied for use with Inform 6
 !
-!  (c) Graham Nelson 1993, 1994, 1995, 1996, 1997
+!  (c) Graham Nelson 1993, 1994, 1995, 1996, 1997, 1998, 1999
 !      but freely usable (see manuals)
 ! ----------------------------------------------------------------------------
-
-Constant LibSerial  = "970405";
-Constant LibRelease = "6/5";
+System_file;
+IFDEF INFIX; IFNDEF DEBUG; Constant DEBUG; ENDIF; ENDIF;
+IFDEF STRICT_MODE; IFNDEF DEBUG; Constant DEBUG; ENDIF; ENDIF;
+Constant LibSerial  = "990428";
+Constant LibRelease = "6/9";
 Constant Grammar__Version = 2;
 IFNDEF VN_1610;
-Message fatalerror "*** Library 6/5 needs Inform v6.10 or later to work ***";
+Message fatalerror "*** Library 6/9 needs Inform v6.10 or later to work ***";
 ENDIF;
-System_file;
 Include "linklpa";
 Fake_Action LetGo;
 Fake_Action Receive;
@@ -30,7 +31,6 @@ Fake_Action Places;
 Fake_Action Objects;
 ENDIF;
 [ Main; InformLibrary.play(); ];
-
 IFDEF USE_MODULES;
 Link "parserm";
 IFNOT;
